#include "../../std.h"

#include "service.h"
#include "backend_file.h"
#include "flo.h"

FlOService::FlOService(const std::string& url,const std::string& path):
                                                 url_(url),storage(path) {
  storage.Read(*this);
}

void FlOService::notify(const FlOMessage& m) {
  // TODO lists must be hashed and protected
  for(list<pFlOServiceRemote>::iterator i = services.begin();i!=services.end();++i) {
    pFlOServiceRemote p = *i;
    p->notify(m);
  };
}

// Process received notification
void FlOService::notification(FlOMessage& msg,pFlOServiceRemote& srv) {
  odlog(3)<<"FlOService: notification: object: "<<msg.object()->id()<<endl;
  // Find object with corresponding id
  // TODO lists must be hashed and protected
  for(list<pFlOObject>::iterator i = objects.begin();i!=objects.end();++i) {
    pFlOObject p = *i;
    odlog(3)<<"FlOService: notification: compare to: "<<p->id()<<endl;
    if(msg == p->id()) {
      p->notification(msg,srv);
      return;
    };
  };
  const_pFlOObject p = msg.object();
  odlog(3)<<"FlOService: notification: static notification"<<endl;
  p->notification(*this,msg,srv);
}

void FlOService::add(pFlOObject o) {
  odlog(3)<<"FlOService: add: object"<<endl;
  if(o->service(this) != this) {
    odlog(1)<<"FlOService: add: failed to reassign object"<<endl;
    return;
  };
  objects.push_back(o);
  storage.Store(o);
  // Notify registered services
  odlog(3)<<"FlOService: add: notifying neighbours"<<endl;
  notify(FlOMessage(*o,FlOObject::created));
}

void FlOService::add(pFlOServiceRemote s) {
  services.push_back(s);
}

pFlOServiceRemote FlOService::service(const std::string& id) {
  for(list<pFlOServiceRemote>::iterator i = services.begin();i!=services.end();++i) {
    pFlOServiceRemote p = *i;
    if(p->id() == id) return *i;
  };
  return new FlOServiceRemote("",id);
}

void FlOService::registr(pFlOServiceRemote s) {
  s->registr(*this);
}

pFlOObject FlOService::find(std::string id) {
  for(list<pFlOObject>::iterator i = objects.begin();i!=objects.end();++i) {
    pFlOObject p = *i;
    if((*i)->id() == id) return (*i);
  };
  return pFlOObject();
}

void FlOService::maintain(void) {
  odlog(3)<<"FlOService: maintain"<<endl;
  for(list<pFlOObject>::iterator i = objects.begin();i!=objects.end();++i) {
    odlog(3)<<"FlOService: maintain: object: "<<(*i)->id()<<endl;
    (*i)->maintain();
  };
  for(list<pFlOServiceRemote>::iterator i = services.begin();i!=services.end();++i) {
    odlog(3)<<"FlOService: maintain: service"<<endl;
    (*i)->maintain();
  };
}

